import model.Strategy
import utils.StringUtils
import java.io.IOException
import java.io.InputStreamReader
import java.io.OutputStreamWriter
import java.net.Socket
import java.util.*


class Runner(private val socket: Socket, private val strategy: Strategy, private val token:String)
{
    private val input = InputStreamReader(socket.getInputStream())
    private val output = OutputStreamWriter(socket.getOutputStream())

    private fun disconnect()
    {
        try {
            socket.close()
        }
        catch(e: IOException) {
            e.printStackTrace()
        }
    }

    fun run()
    {
        try {
            output.write(token)
            output.flush()

            var c: Int
            while (input.read().also { c = it } != -1) {
                when (c.toChar()) {
                    'D' -> disconnect()
                    'A' -> {
                        val player = StringUtils.readPlayer(input)
                        val world = StringUtils.readWorld(input)
                        val a = strategy.getAction(player, world)
                        StringUtils.writeToken(output, a)
                        output.flush()
                    }
                    'I' -> {
                        val rules = StringUtils.readRules(input)
                        strategy.init(rules)
                    }
                    else -> throw InputMismatchException("Unknown action $c")
                }
            }
        }
        catch(e:Exception) {
            e.printStackTrace()
            disconnect()
        }
    }
}

fun main(args: Array<String>) {
    val host = if (args.size < 1) "127.0.0.1" else args[0]
    val port = if (args.size < 2) 7828 else args[1].toInt()
    val token = if (args.size < 3) "0000000000000000000000000000000000000000000000000000000000000000" else args[2]

    val socket = Socket(host, port)
    val runner = Runner(socket, MyStrategy(), token)
    runner.run()
}