package model;

public class Player extends WorldObject {

	public final int playerId;
	public final String playerName;
	private int livesLeft;
	private int score;
	private boolean hasCrashed;
	private int stunDuration;
	private int immuneDuration;
	private int bombsPower, bombsCount, bombsLeft;
	
	public Player(int playerId, String playerName, double x, double y, double playerSize, int livesLeft, int score, boolean hasCrashed, 
			int stunDuration, int immuneDuration, int bombsPower, int bombsCount, int bombsLeft)
	{
		super(WorldObjectType.Player, x, y, playerSize);
		this.playerId = playerId;
		this.playerName = playerName;
		this.livesLeft = livesLeft;
		this.score = score;
		this.hasCrashed = hasCrashed;
		this.stunDuration = stunDuration;
		this.immuneDuration = immuneDuration;
		this.setBombsPower(bombsPower);
		this.setBombsCount(bombsCount);
		this.setBombsLeft(bombsLeft);
	}
	
	public Player clone()
	{
		return new Player(playerId, playerName, getX(), getY(), size, livesLeft, score, hasCrashed, stunDuration, immuneDuration, getBombsPower(), getBombsCount(), getBombsLeft());
	}

	public void move(double dx, double dy)
	{
		this.setX(this.getX() + dx);
		this.setY(this.getY() + dy);
	}

	public int getScore()
	{
		return score;
	}

	public void increaseScore(int score)
	{
		this.score += score;
	}
	
	public void setScore(int score)
	{
		this.score = score;
	}

	public int getStunDuration()
	{
		return stunDuration;
	}

	public void setStunDuration(int stunDuration)
	{
		this.stunDuration = stunDuration;
	}
	
	public void increaseStunDuration(int stunDuration)
	{
		this.stunDuration += stunDuration;
	}

	public int getImmuneDuration()
	{
		return immuneDuration;
	}

	public void setImmuneDuration(int immuneDuration)
	{
		this.immuneDuration = immuneDuration;
	}
	
	public void increaseImmuneDuration(int immuneDuration)
	{
		this.immuneDuration += immuneDuration;
	}

	public boolean getHasCrashed()
	{
		return hasCrashed;
	}

	public void setHasCrashed(boolean hasCrashed)
	{
		this.hasCrashed = hasCrashed;
	}
	
	public boolean isImmune()
	{
		return immuneDuration > 0 || livesLeft <= 0;
	}
	
	public boolean isStunned()
	{
		return stunDuration > 0 || livesLeft <= 0;
	}

	public boolean canPerformActions()
	{
		return !hasCrashed && ! isStunned(); 
	}

	public void tickDurations()
	{
		if(this.getImmuneDuration() > 0)
			this.increaseImmuneDuration(-1);
		if(this.getStunDuration() > 0)
			this.increaseStunDuration(-1);
	}

	public int getBombsPower()
	{
		return bombsPower;
	}

	public void setBombsPower(int bombsPower)
	{
		this.bombsPower = bombsPower;
	}

	public int getBombsCount()
	{
		return bombsCount;
	}

	public void setBombsCount(int bombsCount)
	{
		this.bombsCount = bombsCount;
	}

	public int getBombsLeft()
	{
		return bombsLeft;
	}

	public void setBombsLeft(int bombsLeft)
	{
		this.bombsLeft = bombsLeft;
	}

	public void increaseBombsPower(int bombIncrease)
	{
		this.bombsPower += bombIncrease;
	}
	
	public void increaseBombsCount(int bombIncrease)
	{
		this.bombsCount += bombIncrease;
	}
	
	public void increaseBombsLeft(int bombIncrease)
	{
		this.bombsLeft += bombIncrease;
	}

	public int getLivesLeft()
	{
		return livesLeft;
	}

	public void setLivesLeft(int livesLeft)
	{
		this.livesLeft = livesLeft;
	}
}
