package utils;

import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.Socket;
import java.util.InputMismatchException;

import model.*;

public class RemoteClient extends Client {

	private Socket socket;
	private InputStreamReader in;
	private OutputStreamWriter out;
	
	public RemoteClient(Socket socket) throws IOException
	{
		super();
		this.socket = socket;
		if(this.socket != null)
		{
			this.in = new InputStreamReader(socket.getInputStream());
			this.out = new OutputStreamWriter(socket.getOutputStream());
		}
	}
	
	
	
	public RemoteClient(Socket socket, InputStreamReader in, OutputStreamWriter out)
	{
		super();
		this.socket = socket;
		this.in = in;
		this.out = out;
	}

	@Override
	public void init(Rules rules)
	{
		if(socket != null)
		{
			try
			{
				out.append('I');
				StringUtils.writeToken(out, rules);
				out.flush();
			} catch (IOException e)
			{
				e.printStackTrace();
			}
		}
	}

	@Override
	public void disconnect()
	{
		if(socket!=null)
		{
			try
			{
				out.append('D');
				socket.close();
			} catch (IOException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		socket = null;
	}

	@Override
	public Action getAction(Player player, World world)
	{
		if(socket != null)
		{
			try
			{
				startClock();
				out.write('A');
				StringUtils.writeToken(out, player);
				StringUtils.writeToken(out, world);
				out.flush();
				Action a = StringUtils.readAction(in);
				stopClock();
				return a;
			} catch (IOException | InputMismatchException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
				disconnect();
				
			}
		}
		return null;
	}

}
