package utils;

import java.io.IOException;
import java.io.Reader;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Collection;
import java.util.InputMismatchException;
import java.util.List;
import java.util.TreeSet;

import model.*;

public class StringUtils {
	public static String escapeString(String x)
	{
		StringBuilder res = new StringBuilder();
		for(char c: x.toCharArray())
		{
			if( c == '\\' )
				res.append("\\\\");
			else if( c == '[' )
				res.append("\\(");
			else if( c == ']' )
				res.append("\\)");
			else
				res.append(c);
		}
		return res.toString();
	}
	public static String unescapeString(String x) throws InputMismatchException
	{
		StringBuilder res = new StringBuilder();
		for(int i=0;i<x.length();i++)
		{
			char c = x.charAt(i);
			if( c == '\\')
			{
				if( i == x.length()-1)
					throw new InputMismatchException("Last character of string is '\\'");
				else if( x.charAt(i+1) == '\\' )
					res.append('\\');
				else if( x.charAt(i+1) == '(')
					res.append('[');
				else if( x.charAt(i+1) == ')')
					res.append(']');
				else
					throw new InputMismatchException("Unrecognized sequence " + c + x.charAt(i+1));
				i++;
			}
			else
				res.append(c);
		}
		return res.toString();
	}
	public static void writeToken(Writer out, String x) throws IOException
	{
		out.write("[");
		out.write(escapeString(x));
		out.write("]");
	}
	public static void writeToken(Writer out, int x) throws IOException
	{
		writeToken(out, Integer.toString(x));
	}
	public static void writeToken(Writer out, double x) throws IOException
	{
		writeToken(out, Double.toString(x));
	}
	public static void writeToken(Writer out, boolean x) throws IOException
	{
		writeToken(out, Boolean.toString(x));
	}
	
	public static void writeToken(Writer out, WorldObjectType x) throws IOException
	{
		writeToken(out, x.id);
	}
	
	public static void writeToken(Writer out, BonusType x) throws IOException
	{
		writeToken(out, x.id);
	}
	
	public static void writeToken(Writer out, Action x) throws IOException
	{
		writeToken(out, x.speedX);
		writeToken(out, x.speedY);
		writeToken(out, x.drop);
	}
	
	public static void writeToken(Writer out, WorldObject x) throws IOException
	{
		writeToken(out, x.type);
		writeToken(out, x.getX());
		writeToken(out, x.getY());
		writeToken(out, x.size);
	}
	
	public static void writeToken(Writer out, Bomb x) throws IOException
	{
		writeToken(out, x.getX());
		writeToken(out, x.getY());
		writeToken(out, x.size);
		writeToken(out, x.timeout);
		writeToken(out, x.getOwner());
		writeToken(out, x.getWalkableFor());
	}
	
	public static void writeToken(Writer out, Bonus x) throws IOException
	{
		writeToken(out, x.getX());
		writeToken(out, x.getY());
		writeToken(out, x.size);
		writeToken(out, x.getImmuneDuration());
		writeToken(out, x.bonusType);
	}
	
	public static void writeToken(Writer out, Explosion x) throws IOException
	{
		writeToken(out, x.getX());
		writeToken(out, x.getY());
		writeToken(out, x.size);
		writeToken(out, x.getTimeout());
		writeToken(out, x.getCreators());
	}
	
	public static void writeToken(Writer out, Obstacle x) throws IOException
	{
		writeToken(out, x.type);
		writeToken(out, x.getX());
		writeToken(out, x.getY());
		writeToken(out, x.size);
		writeToken(out, x.getContents());
	}
	
	public static void writeToken(Writer out, Player x) throws IOException
	{
		writeToken(out, x.playerId);
		writeToken(out, x.playerName);
		writeToken(out, x.getX());
		writeToken(out, x.getY());
		writeToken(out, x.size);
		writeToken(out, x.getLivesLeft());
		writeToken(out, x.getScore());
		writeToken(out, x.getHasCrashed());
		writeToken(out, x.getStunDuration());
		writeToken(out, x.getImmuneDuration());
		writeToken(out, x.getBombsPower());
		writeToken(out, x.getBombsCount());
		writeToken(out, x.getBombsLeft());
	}
	
	public static void writeToken(Writer out, Collection<Integer> x) throws IOException
	{
		out.write("{");
		for(var obj: x)
		{
			out.write(">");
			writeToken(out, obj);
		}
		out.write("}");
	}
	
	public static void writeToken(Writer out, WorldObject[] x) throws IOException
	{
		out.write("{");
		for(var obj: x)
		{
			out.write(">");
			writeToken(out, obj);
		}
		out.write("}");
	}
	
	public static void writeToken(Writer out, Obstacle[] x) throws IOException
	{
		out.write("{");
		for(var obj: x)
		{
			out.write(">");
			writeToken(out, obj);
		}
		out.write("}");
	}
	
	public static void writeToken(Writer out, Bomb[] x) throws IOException
	{
		out.write("{");
		for(var obj: x)
		{
			out.write(">");
			writeToken(out, obj);
		}
		out.write("}");
	}
	
	public static void writeToken(Writer out, Bonus[] x) throws IOException
	{
		out.write("{");
		for(var obj: x)
		{
			out.write(">");
			writeToken(out, obj);
		}
		out.write("}");
	}
	
	public static void writeToken(Writer out, Explosion[] x) throws IOException
	{
		out.write("{");
		for(var obj: x)
		{
			out.write(">");
			writeToken(out, obj);
		}
		out.write("}");
	}
	
	public static void writeToken(Writer out, Player[] x) throws IOException
	{
		out.write("{");
		for(var obj: x)
		{
			out.write(">");
			writeToken(out, obj);
		}
		out.write("}");
	}
	
	public static void writeToken(Writer out, World x) throws IOException
	{
		writeToken(out, x.tick);
		writeToken(out, x.getPlayers());
		writeToken(out, x.getObstacles());
		writeToken(out, x.getBombs());
		writeToken(out, x.getExplosions());
		writeToken(out, x.getBonuses());
	}
	
	public static void writeToken(Writer out, Rules r) throws IOException
	{
		writeToken(out, r.PLAYER_COUNT);
		
		writeToken(out, r.PLAYER_SIZE);
		writeToken(out, r.PLAYER_LIVES_COUNT);
		writeToken(out, r.PLAYER_MAX_SPEED);
		writeToken(out, r.PLAYER_AFTER_HIT_IMMUNE_DURATION);
		writeToken(out, r.PLAYER_AFTER_HIT_STUN_DURATION);

		writeToken(out, r.BOMB_SIZE);
		writeToken(out, r.BOMB_DELAY);
		writeToken(out, r.BOMBS_INITIAL_POWER);
		writeToken(out, r.BOMBS_INITIAL_AMOUNT);

		writeToken(out, r.BONUS_SIZE);
		writeToken(out, r.BONUS_IMMUNE_DURATION);
		writeToken(out, r.BONUS_BOMB_COUNT);
		writeToken(out, r.BONUS_POWER_COUNT);

		writeToken(out, r.EXPLOSION_SIZE);
		writeToken(out, r.EXPLOSION_DURATION);
		writeToken(out, r.BRICK_SIZE);
		writeToken(out, r.BRICK_COUNT);

		writeToken(out, r.WORLD_WIDTH_IN_BRICKS);
		writeToken(out, r.WORLD_HEIGHT_IN_BRICKS);

		writeToken(out, r.WALL_HIT_SCORE);
		writeToken(out, r.PLAYER_HIT_SCORE);
		writeToken(out, r.PLAYER_SELF_HIT_PENALTY);
		writeToken(out, r.BONUS_PICKUP_SCORE);

		writeToken(out, r.WIDTH);
		writeToken(out, r.HEIGHT);

		writeToken(out, r.GAME_DURATION);
	}
	
	public static String readString(Reader in) throws IOException, InputMismatchException
	{
		int x;
		x = in.read();
		if( x != '[' )
			throw new InputMismatchException();
		
		StringBuilder builder = new StringBuilder();
		
		while( ( x = in.read() ) != -1)
		{
			if ( x == ']' ) break;
			builder.append((char)x);
		}
		if( x != ']' )
		{
			if( x == -1)
				throw new InputMismatchException("Expected ] but stream ended unexpectedly");
			else
				throw new InputMismatchException("Expected ] but " + (char)x + " found");
		}
		
		return unescapeString(builder.toString());
	}
	
	public static int readInteger(Reader in) throws IOException, InputMismatchException
	{
		try
		{
			return Integer.parseInt(readString(in));
		}
		catch (NumberFormatException e)
		{
			throw new InputMismatchException(e.getMessage());
		}
	}
	
	public static double readDouble(Reader in) throws IOException, InputMismatchException
	{
		try
		{
			return Double.parseDouble(readString(in));
		}
		catch (NumberFormatException e)
		{
			throw new InputMismatchException(e.getMessage());
		}
	}
	
	public static boolean readBoolean(Reader in) throws IOException, InputMismatchException
	{
		String s = readString(in);
		if ( s.equals("true") )
			return true;
		else if( s.equals("false") )
			return false;
		else
			throw new InputMismatchException();
	}
	
	public static Action readAction(Reader in) throws IOException, InputMismatchException
	{
		Action action = new Action();
		action.speedX = readDouble(in);
		action.speedY = readDouble(in);
		action.drop = readBoolean(in);
		
		return action;
	}
	
	public static WorldObjectType readWorldObjectType(Reader in) throws IOException, InputMismatchException
	{
		int x = readInteger(in);
		if( 0<=x && x < WorldObjectType.count)
			return WorldObjectType.getById(x);
		else
			throw new InputMismatchException();
	}
	
	public static BonusType readBonusType(Reader in) throws IOException, InputMismatchException
	{
		int x = readInteger(in);
		if( 0<=x && x < BonusType.count)
			return BonusType.getById(x);
		else
			throw new InputMismatchException();
	}

	public static WorldObject readWorldObject(Reader in) throws IOException, InputMismatchException
	{
		WorldObjectType type = readWorldObjectType(in);
		double x = readDouble(in);
		double y = readDouble(in);
		double sz = readDouble(in);
		
		return new WorldObject(type, x, y, sz);
	}
	
	public static Obstacle readObstacle(Reader in) throws IOException, InputMismatchException
	{
		WorldObjectType type = readWorldObjectType(in);
		double x = readDouble(in);
		double y = readDouble(in);
		double size = readDouble(in);
		BonusType contents = readBonusType(in);
		
		return new Obstacle(type, x, y, size, contents);
	}
	
	public static Bomb readBomb(Reader in) throws IOException, InputMismatchException
	{
		double x = readDouble(in);
		double y = readDouble(in);
		double size = readDouble(in);
		int timeout = readInteger(in);
		int owner = readInteger(in);
		List<Integer> walkableFor = readIntegers(in);
		
		return new Bomb(x, y, size, timeout, owner, walkableFor);
	}

	public static Bonus readBonus(Reader in) throws IOException, InputMismatchException
	{
		double x = readDouble(in);
		double y = readDouble(in);
		double size = readDouble(in);
		int immuneDuration = readInteger(in);
		BonusType bonusType = readBonusType(in);
		
		return new Bonus(x, y, size, bonusType, immuneDuration);
	}
	
	public static Explosion readExplosion(Reader in) throws IOException, InputMismatchException
	{
		double x = readDouble(in);
		double y = readDouble(in);
		double size = readDouble(in);
		int timeout = readInteger(in);
		List<Integer> creators = readIntegers(in);
		
		return new Explosion(x, y, size, timeout, new TreeSet<Integer>(creators));
	}
	
	public static Player readPlayer(Reader in) throws IOException, InputMismatchException
	{
		int playerId = readInteger(in);
		String playerName = readString(in);
		double x = readDouble(in);
		double y = readDouble(in);
		double size = readDouble(in);
		int livesLeft = readInteger(in);
		int score = readInteger(in);
		boolean hasCrashed = readBoolean(in);
		int stunDuration = readInteger(in);
		int immuneDuration = readInteger(in);
		int bombsPower = readInteger(in);
		int bombsCount = readInteger(in);
		int bombsLeft = readInteger(in);
		
		return new Player(playerId, playerName, x, y, size, livesLeft, score, hasCrashed, stunDuration, immuneDuration, bombsPower, bombsCount, bombsLeft);
	}

	public static List<Integer> readIntegers(Reader in) throws IOException
	{
		int c;
		c = in.read();
		if( c != '{' )
			throw new InputMismatchException();
		
		ArrayList<Integer> res = new ArrayList<>();
		
		while( ( c = in.read() ) == '>' )
			res.add(readInteger(in));
	
		if( c != '}' )
			throw new InputMismatchException();
		
		return res;
	}
	
	public static List<WorldObject> readWorldObjects(Reader in) throws IOException
	{
		int c;
		c = in.read();
		if( c != '{' )
			throw new InputMismatchException();
		
		ArrayList<WorldObject> res = new ArrayList<>();
		
		while( ( c = in.read() ) == '>' )
			res.add(readWorldObject(in));
	
		if( c != '}' )
			throw new InputMismatchException();
		
		return res;
	}
	
	public static List<Obstacle> readObstacles(Reader in) throws IOException
	{
		int c;
		c = in.read();
		if( c != '{' )
			throw new InputMismatchException();
		
		ArrayList<Obstacle> res = new ArrayList<>();
		
		while( ( c = in.read() ) == '>' )
			res.add(readObstacle(in));
	
		if( c != '}' )
			throw new InputMismatchException();
		
		return res;
	}
	
	public static List<Bomb> readBombs(Reader in) throws IOException
	{
		int c;
		c = in.read();
		if( c != '{' )
			throw new InputMismatchException();
		
		ArrayList<Bomb> res = new ArrayList<>();
		
		while( ( c = in.read() ) == '>' )
			res.add(readBomb(in));
	
		if( c != '}' )
			throw new InputMismatchException();
		
		return res;
	}
	
	public static List<Bonus> readBonuses(Reader in) throws IOException
	{
		int c;
		c = in.read();
		if( c != '{' )
			throw new InputMismatchException();
		
		ArrayList<Bonus> res = new ArrayList<>();
		
		while( ( c = in.read() ) == '>' )
			res.add(readBonus(in));
	
		if( c != '}' )
			throw new InputMismatchException();
		
		return res;
	}

	public static List<Explosion> readExplosions(Reader in) throws IOException
	{
		int c;
		c = in.read();
		if( c != '{' )
			throw new InputMismatchException();
		
		ArrayList<Explosion> res = new ArrayList<>();
		
		while( ( c = in.read() ) == '>' )
			res.add(readExplosion(in));
	
		if( c != '}' )
			throw new InputMismatchException();
		
		return res;
	}
	
	public static List<Player> readPlayers(Reader in) throws IOException
	{
		int c;
		c = in.read();
		if( c != '{' )
			throw new InputMismatchException();
		
		ArrayList<Player> res = new ArrayList<>();
		
		while( ( c = in.read() ) == '>' )
			res.add(readPlayer(in));
	
		if( c != '}' )
			throw new InputMismatchException();
		
		return res;
	}

	public static World readWorld(Reader in) throws IOException, InputMismatchException
	{
		int tick = readInteger(in);
		List<Player> players = readPlayers(in);
		List<Obstacle> obstacles = readObstacles(in);
		List<Bomb> bombs = readBombs(in);
		List<Explosion> explosions = readExplosions(in);
		List<Bonus> bonuses = readBonuses(in);
		
		return new World(tick, players, obstacles, bombs, explosions, bonuses);
	}
	
	public static Rules readRules(Reader in) throws IOException, InputMismatchException
	{
		int PLAYER_COUNT = readInteger(in);

		double PLAYER_SIZE = readDouble(in);
		int PLAYER_LIVES_COUNT = readInteger(in);
		double PLAYER_MAX_SPEED = readDouble(in);
		int PLAYER_AFTER_HIT_IMMUNE_DURATION = readInteger(in);
		int PLAYER_AFTER_HIT_STUN_DURATION = readInteger(in);

		double BOMB_SIZE = readDouble(in);
		int BOMB_DELAY = readInteger(in);
		int BOMBS_INITIAL_POWER = readInteger(in);
		int BOMBS_INITIAL_AMOUNT = readInteger(in);

		int BONUS_SIZE = readInteger(in);
		int BONUS_IMMUNE_DURATION = readInteger(in);
		int BONUS_BOMB_COUNT = readInteger(in);
		int BONUS_POWER_COUNT = readInteger(in);

		double EXPLOSION_SIZE = readDouble(in);
		int EXPLOSION_DURATION = readInteger(in);
		double BRICK_SIZE = readDouble(in);
		int BRICK_COUNT = readInteger(in);

		int WORLD_WIDTH_IN_BRICKS = readInteger(in);
		int WORLD_HEIGHT_IN_BRICKS = readInteger(in);

		int WALL_HIT_SCORE = readInteger(in);
		int PLAYER_HIT_SCORE = readInteger(in);
		int PLAYER_SELF_HIT_PENALTY = readInteger(in);
		int BONUS_PICKUP_SCORE = readInteger(in);

		double WIDTH = readDouble(in);
		double HEIGHT = readDouble(in);

		int GAME_DURATION = readInteger(in);
		
		return new Rules(PLAYER_COUNT, PLAYER_SIZE, PLAYER_LIVES_COUNT, PLAYER_MAX_SPEED,
				PLAYER_AFTER_HIT_IMMUNE_DURATION, PLAYER_AFTER_HIT_STUN_DURATION, BOMB_SIZE, BOMB_DELAY,
				BOMBS_INITIAL_POWER, BOMBS_INITIAL_AMOUNT, BONUS_SIZE, BONUS_IMMUNE_DURATION,
				BONUS_BOMB_COUNT, BONUS_POWER_COUNT, EXPLOSION_SIZE, EXPLOSION_DURATION,
				BRICK_SIZE, BRICK_COUNT, WORLD_WIDTH_IN_BRICKS, WORLD_HEIGHT_IN_BRICKS,
				WALL_HIT_SCORE, PLAYER_HIT_SCORE, PLAYER_SELF_HIT_PENALTY, BONUS_PICKUP_SCORE, WIDTH,
				HEIGHT, GAME_DURATION);
	}
}
